#include "paths/paths.h"
#include "butterflies_populations.h"
#include "modules/module_header.h"
#include "actions/cats_actions.h"

#include "butterflies_overlays.h"
#include "butterflies_main.h"
#include "inline_overlays.h"
#include "inline_population.h"


enum action_status bf_grid_overlay_update(const struct cats_configuration *conf, struct cats_grid *grid)
{
        if (conf->overlays.have_overlays == false) {
                return ACTION_NOT_RUN;
        }

        int module_id = CATS_MODULE_ID;
        struct grid_data_butterflies *data = grid->grid_modules[module_id].module_data;


        const cats_dt_coord rows = grid->dimension.rows;
        const cats_dt_coord cols = grid->dimension.cols;
        int64_t cells_habitat_ok = 0;
        int64_t cells_excluded = 0;
        int64_t cells_resource_ok = 0;
        int64_t cells_habitat_and_resource_ok = 0;

        for (cats_dt_coord row = 0; row < rows; row++) {
                for (cats_dt_coord col = 0; col < cols; col++) {

                        data->info_layer[row][col] = BF_CELL_CLEAR;

                        if (cell_excluded_by_overlay(conf, row, col)) {
                                data->info_layer[row][col] |= BF_CELL_EXCLUDED;
                                data->eggs[row][col] = 0;
                                set_population_ignore_cc(grid, row, col, 0);
                                cells_excluded += 1;
                                continue;
                        }


                        if (conf->overlays.overlay[OL_HABITAT_TYPE_CC].enabled &&
                            conf->overlays.habitat_cc->data[row][col] > 0) {
                                data->info_layer[row][col] |= BF_CELL_HABITAT_OK;
                                cells_habitat_ok += 1;
                        }

                        if (conf->overlays.overlay[OL_RESOURCE].enabled &&
                            conf->overlays.resources->data[row][col] > 0) {
                                data->info_layer[row][col] |= BF_CELL_RESOURCE_AVAILABLE;
                                cells_resource_ok += 1;
                        }

                        if ((data->info_layer[row][col] & BF_CELL_HABITAT_OK) &&
                                (data->info_layer[row][col] & BF_CELL_RESOURCE_AVAILABLE)) {
                                data->info_layer[row][col] |= BF_CELL_VALID_DISPERSAL_TARGET;
                                cells_habitat_and_resource_ok += 1;

                        } else {
                                data->eggs[row][col] = 0;
                                set_population_ignore_cc(grid, row, col, 0);
                        }
                }
        }

        int64_t cells_with_eggs = 0;
        double total_eggs = 0;
        for (cats_dt_coord row = 0; row < rows; row++) {
                for (cats_dt_coord col = 0; col < cols; col++) {
                        float eggs = data->eggs[row][col];
                        if (eggs > 0) {
                                cells_with_eggs += 1;
                                total_eggs += eggs;
                        }

                }
        }

        log_message(LOG_INFO, "Overlay update: %ld excluded, %ld habitat ok, %ld resource ok, %ld habitat + resource ok",
                    cells_excluded, cells_habitat_ok, cells_resource_ok, cells_habitat_and_resource_ok);
        log_message(LOG_INFO, "Overlay update %ld cells with eggs, total %f eggs, average %f eggs per cell", cells_with_eggs, total_eggs, total_eggs/(float) cells_with_eggs);
        return ACTION_RUN;
}
